/* linuxsyslogd: read messages from /dev/log, decode the prefixes and
   write them to stderr, in a suitable format for multilog or splogger
   to deal with. (Admittedly, using splogger with this would be silly.)

   Copyright (c) 2000, 2003 Adam Sampson, azz@us-lot.org

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
   files (the "Software"), to deal in the Software without
   restriction, including without limitation the rights to use, copy,
   modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:
  
   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.
   
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
   WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <sys/errno.h>
#include <sys/syslog.h>
#include <string.h>
#include <unistd.h>

#define LOGPATH "/dev/log"
#define WRITEMSG(msg) { char s[] = msg; write(2, s, (sizeof s) - 1); }

int main(int argc, char **argv) {
  struct sockaddr_un sa;
  int logfd;
#define LOGBUFSIZE 16384
  char logbuf[LOGBUFSIZE];
  
  /* Build a sockaddr_un structure for /dev/log. */
  memset(&sa, 0, sizeof(sa));
  sa.sun_family = AF_UNIX;
  strncpy(sa.sun_path, LOGPATH, sizeof(sa.sun_path));
  
  /* Remove the existing socket. (Don't worry if it can't be found;
     syslogd removes it on exit.) */
  if (unlink(LOGPATH) < 0 && errno != ENOENT) {
    WRITEMSG("linuxsyslogd: unable to unlink socket\n");
    return 20;
  }
  
  /* Open the socket. */
  logfd = socket(AF_UNIX, SOCK_DGRAM, 0);
  if (logfd < 0) {
    WRITEMSG("linuxsyslogd: unable to open socket\n");
    return 21;
  }
  
  /* Bind to the socket. */
  if (bind(logfd, (struct sockaddr *) &sa, 
	   sizeof(sa.sun_family) + strlen(sa.sun_path)) < 0) {
    WRITEMSG("linuxsyslogd: unable to bind to socket\n");
    return 22;
  }
  
  /* Change the mode of the socket so that anyone can write to it. */
  if (chmod(LOGPATH, 0666) < 0) {
    WRITEMSG("linuxsyslogd: unable to chmod socket\n");
    return 23;
  }

  while (1) {
    int first = 0, pos = 0, len = 0, bufcount;

    /* Read data from the socket. */
    bufcount = recv(logfd, logbuf, LOGBUFSIZE - 1, 0);
    if (bufcount < 0) {
      if (errno == EINTR) continue;
      WRITEMSG("linuxsyslogd: unable to read from socket\n");
      return 24;
    }

    /* Iterate through the received message, splitting it up into
       individual messages. */
    while (pos <= bufcount) {

      if (logbuf[pos] == '\0' || pos == bufcount) {
	len = pos - first;

	/* Attempt to convert a prefix if one exists. The prefix looks
	   like <xxx>, where xxx is the syslog facility and priority code
	   as a decimal number. */
	if (len > 0 && logbuf[first] == '<') {
	  int val = 0, loc = 1;
	  while (loc < len 
		 && logbuf[first + loc] >= '0' 
		 && logbuf[first + loc] <= '9') {
	    val = (10 * val) + (logbuf[first + loc] - '0');
	    ++loc;
	  }
	  if (loc < len && logbuf[first + loc] == '>') loc++;

	  switch (LOG_FAC(val)) {
	  case 0: WRITEMSG("kern"); break;
	  case 1: WRITEMSG("user"); break;
	  case 2: WRITEMSG("mail"); break;
	  case 3: WRITEMSG("daemon"); break;
	  case 4: WRITEMSG("auth"); break;
	  case 5: WRITEMSG("syslog"); break;
	  case 6: WRITEMSG("lpr"); break;
	  case 7: WRITEMSG("news"); break;
	  case 8: WRITEMSG("uucp"); break;
	  case 9: WRITEMSG("cron"); break;
	  case 10: WRITEMSG("authpriv"); break;
	  case 11: WRITEMSG("ftp"); break;
	  case 16: WRITEMSG("local0"); break;
	  case 17: WRITEMSG("local1"); break;
	  case 18: WRITEMSG("local2"); break;
	  case 19: WRITEMSG("local3"); break;
	  case 20: WRITEMSG("local4"); break;
	  case 21: WRITEMSG("local5"); break;
	  case 22: WRITEMSG("local6"); break;
	  case 23: WRITEMSG("local7"); break;
	  default: goto noprefix;
	  }

	  WRITEMSG(".");

	  switch (LOG_PRI(val)) {
	  case 0: WRITEMSG("emerg"); break;
	  case 1: WRITEMSG("alert"); break;
	  case 2: WRITEMSG("crit"); break;
	  case 3: WRITEMSG("err"); break;
	  case 4: WRITEMSG("warning"); break;
	  case 5: WRITEMSG("notice"); break;
	  case 6: WRITEMSG("info"); break;
	  case 7: WRITEMSG("debug"); break;
	  default: goto noprefix;
	  }

	  WRITEMSG(": ");

	  first += loc;
	  len -= loc;
	}
      noprefix:

	/* Write the message to stderr. */
	write(2, logbuf + first, len);
	WRITEMSG("\n");

	first = pos + 1;
      }
      ++pos;
    }
  }
    
  return 0; /* NOTREACHED */
}
